/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Alec Flett <alecf@netscape.com>
 *   Scott Putterman <putterman@netscape.com>
 *   Seth Spitzer <sspitzer@netscape.com>
 *   Pierre Phaneuf <pp@ludusdesign.com>
 *   David Bienvenu <bienvenu@nventure.com>
 *   Postbox, Inc. <contact@postbox-inc.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsMessengerBootstrap.h"
#include "nsCOMPtr.h"

#include "nsDOMCID.h"
#include "nsMsgBaseCID.h"
#include "nsIMsgMailSession.h"
#include "nsIMsgFolderCache.h"
#include "nsIPrefService.h"
#include "nsIPrefBranch.h"
#include "nsIDOMWindow.h"
#include "nsXPCOM.h"
#include "nsISupportsPrimitives.h"
#include "nsIWindowWatcher.h"
#include "nsString.h"
#include "nsIURI.h"
#include "nsIDialogParamBlock.h"
#include "nsUnicharUtils.h"

#include "nsICommandLine.h"
#include "nsILocalFile.h"
#include "nsNetUtil.h"
#include "nsIFileURL.h"
#include "nsNativeCharsetUtils.h"
#include "nsIRDFResource.h"
#include "nsIRDFService.h"
#include "nsIMsgHdr.h"
#include "nsMsgUtils.h"
#include "nsEscape.h"
#include "nsIMsgGlobalSearch.h"

#ifdef XP_MACOSX
#include "nsDirectoryServiceDefs.h"
#endif

#define MAPI_STARTUP_ARG "MapiStartup"

NS_IMPL_THREADSAFE_ADDREF(nsMessengerBootstrap)
NS_IMPL_THREADSAFE_RELEASE(nsMessengerBootstrap)

NS_IMPL_QUERY_INTERFACE2(nsMessengerBootstrap,
                         nsICommandLineHandler,
                         nsIMessengerWindowService)

nsMessengerBootstrap::nsMessengerBootstrap()
{
}

nsMessengerBootstrap::~nsMessengerBootstrap()
{
}

NS_IMETHODIMP
nsMessengerBootstrap::Handle(nsICommandLine* aCmdLine)
{
  nsresult rv;

  nsCOMPtr<nsIWindowWatcher> wwatch (do_GetService(NS_WINDOWWATCHER_CONTRACTID));
  NS_ENSURE_TRUE(wwatch, NS_ERROR_FAILURE);

  nsCOMPtr<nsIDOMWindow> opened;

#ifndef MOZ_SUITE
  PRBool found;
  rv = aCmdLine->HandleFlag(NS_LITERAL_STRING("options"), PR_FALSE, &found);
  if (NS_SUCCEEDED(rv) && found) {
    wwatch->OpenWindow(nsnull, "chrome://messenger/content/preferences/preferences.xul", "_blank",
                      "chrome,dialog=no,all", nsnull, getter_AddRefs(opened));
    aCmdLine->SetPreventDefault(PR_TRUE);
  }
#endif
  
  nsAutoString mailUrl; // -mail or -mail <some url> 
  PRBool flag = PR_FALSE;
  rv = aCmdLine->HandleFlagWithParam(NS_LITERAL_STRING("mail"), PR_FALSE, mailUrl);
  if (NS_SUCCEEDED(rv))
    flag = !mailUrl.IsVoid();
  else 
    aCmdLine->HandleFlag(NS_LITERAL_STRING("mail"), PR_FALSE, &flag);
  if (flag)
  {
    nsCOMPtr<nsISupportsArray> argsArray = do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv);
    NS_ENSURE_SUCCESS(rv, rv);

    // create scriptable versions of our strings that we can store in our nsISupportsArray....
    if (!mailUrl.IsEmpty())
    {
      nsCOMPtr<nsISupportsString> scriptableURL (do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID));
      NS_ENSURE_TRUE(scriptableURL, NS_ERROR_FAILURE);
      if (StringBeginsWith(mailUrl, NS_LITERAL_STRING("mailbox-message://")) ||
          StringBeginsWith(mailUrl, NS_LITERAL_STRING("imap-message://")) ||
          StringBeginsWith(mailUrl, NS_LITERAL_STRING("news-message://")))
      {
        nsCOMPtr <nsIMsgDBHdr> msgHdr;
        nsCAutoString nativeArg;
        NS_CopyUnicodeToNative(mailUrl, nativeArg);
        PRInt32 queryIndex = nativeArg.Find("?messageId=", PR_TRUE);
        if (queryIndex > 0)
        {
          nsCString messageId, folderUri;
          nativeArg.Right(messageId, nativeArg.Length() - queryIndex - 11);
          nativeArg.Left(folderUri, queryIndex);
          folderUri.Cut(folderUri.Find("-message"), 8);
          return OpenMessengerWindowForMessageId(folderUri, messageId);
        }
        else
          GetMsgDBHdrFromURI(nativeArg.get(), getter_AddRefs(msgHdr));

        if (msgHdr)
        {
          nsCOMPtr <nsIMsgFolder> folder;
          nsCString folderUri;
          nsMsgKey msgKey;
          msgHdr->GetMessageKey(&msgKey);
          msgHdr->GetFolder(getter_AddRefs(folder));
          if (folder)
          {
            folder->GetURI(folderUri);
            rv = DiscoverFoldersIfNeeded(folder);
            NS_ENSURE_SUCCESS(rv, rv);
            return OpenMessengerWindowWithUri("mail:messageWindow", folderUri.get(), msgKey);  
          }
        }
      }
      // check if it's a mail message url, and if so, convert it?
      scriptableURL->SetData((mailUrl));
      argsArray->AppendElement(scriptableURL);
    }

    wwatch->OpenWindow(nsnull, "chrome://messenger/content/", "_blank",
                       "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar,dialog=no", argsArray, getter_AddRefs(opened));
    aCmdLine->SetPreventDefault(PR_TRUE);
    return NS_OK;
  }

#ifdef XP_WIN
  // Handle MAPI startup -- do nothing
  PRBool isMapiStartup = PR_FALSE;
  aCmdLine->HandleFlag(NS_LITERAL_STRING(MAPI_STARTUP_ARG), PR_FALSE, &isMapiStartup);
#endif

#define MAPI_STARTUP_ARG "MapiStartup"

#ifndef MOZ_SUITE
  PRInt32 numArgs;
  aCmdLine->GetLength(&numArgs);
  if (numArgs > 0)
  {
    nsString mailPath;
#ifdef XP_MACOSX
    PRInt32 found;
    rv = aCmdLine->FindFlag(NS_LITERAL_STRING("file"), PR_FALSE, &found);
    if (NS_SUCCEEDED(rv) && found == -1)
      rv = aCmdLine->FindFlag(NS_LITERAL_STRING("url"), PR_FALSE, &found);
    if (NS_SUCCEEDED(rv) && found != -1 && numArgs > found + 1)
    {
      aCmdLine->GetArgument(found + 1, mailPath); // pull out the file spec argument if present

      if (!mailPath.IsEmpty() && 
          (StringEndsWith(mailPath, NS_LITERAL_STRING(".pbeml"), nsCaseInsensitiveStringComparator()) ||
           StringBeginsWith(mailPath, NS_LITERAL_STRING("x-postbox-message://"), nsCaseInsensitiveStringComparator())))
      {
        // note, the incoming fileSpec is escaped, we need it unescaped.
        // for compose, we decode this in GetArgs() in MsgComposeCommands.js with decodeURIComponent()
        // for spotlight, we decode in HandleIndexerResult()
        rv = HandleIndexerResult(mailPath);
        if (NS_FAILED(rv))  
        {
          // XXX todo
          // should we delete the .pbeml file as we don't have the message for it?
          // Otherwise, this ghost hit will appear in spotlight
          // (maybe only if it is in ~/Library/Caches/Metadata/Postbox)
        }

        // If we've found a search result (or not), don't pop up a 3pane window
        // we never want to open the compose window with a .pbeml file
        aCmdLine->SetPreventDefault(PR_TRUE);
        return NS_OK;
     }
    }
#else
    // All other OSes just get a straight path (i.e., no -file)
    nsAutoString arg;
    aCmdLine->GetArgument(0, arg);
    mailPath = arg;
#endif
    if (StringEndsWith(mailPath, NS_LITERAL_STRING(".eml"), nsCaseInsensitiveStringComparator()))
    {
      nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
      NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
      rv = file->InitWithPath(mailPath);
      NS_ENSURE_SUCCESS(rv, rv);
      // should we check that the file exists, or looks like a mail message?

      nsCOMPtr<nsIURI> uri;
      NS_NewFileURI(getter_AddRefs(uri), file);
      nsCOMPtr<nsIFileURL> fileURL(do_QueryInterface(uri));
      NS_ENSURE_TRUE(fileURL, NS_ERROR_FAILURE);

      // create scriptable versions of our strings that we can store in our nsISupportsArray....
      nsCOMPtr<nsISupportsString> scriptableURL (do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID));
      NS_ENSURE_TRUE(scriptableURL, NS_ERROR_FAILURE);

      fileURL->SetQuery(NS_LITERAL_CSTRING("?type=application/x-message-display"));

      wwatch->OpenWindow(nsnull, "chrome://messenger/content/messageWindow.xul", "_blank",
                         "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar,dialog=no", fileURL, getter_AddRefs(opened));
      aCmdLine->SetPreventDefault(PR_TRUE);
    }
    return NS_OK;

  }
#endif
  return NS_OK;
}

NS_IMETHODIMP
nsMessengerBootstrap::GetHelpInfo(nsACString& aResult)
{
  aResult.Assign(
    "  -mail                Open the mail folder view.\n"
#ifndef MOZ_SUITE
    "  -options             Open the options dialog.\n"
#endif
  );

  return NS_OK;
}
nsresult nsMessengerBootstrap::DiscoverFoldersIfNeeded(nsIMsgFolder *folder)
{
  nsCOMPtr <nsIMsgFolder> parent;
  folder->GetParent(getter_AddRefs(parent));
  // check if we've done folder discovery. If not,
  // do it so we'll have a real folder.
  if (!parent)
  {
    nsCOMPtr <nsIMsgIncomingServer> server;
    folder->GetServer(getter_AddRefs(server));
    nsresult rv = server->GetRootFolder(getter_AddRefs(parent));
    NS_ENSURE_SUCCESS(rv, rv);
    nsCOMPtr<nsIEnumerator> enumerator;
    parent->GetSubFolders(getter_AddRefs(enumerator));
  }
  return NS_OK;
}

nsresult nsMessengerBootstrap::OpenMessengerWindowForMessageId(nsCString &folderUri, nsCString &messageId)
{
  nsresult rv;
  nsCOMPtr<nsIRDFService> rdf(do_GetService("@mozilla.org/rdf/rdf-service;1", &rv));
  NS_ENSURE_SUCCESS(rv, rv);
  nsCOMPtr<nsIRDFResource> res;
  rv = rdf->GetResource(folderUri, getter_AddRefs(res));
  NS_ENSURE_SUCCESS(rv, rv);
  nsCOMPtr<nsIMsgFolder> containingFolder;
  containingFolder = do_QueryInterface(res, &rv);
  NS_ENSURE_SUCCESS(rv, rv);
  rv = DiscoverFoldersIfNeeded(containingFolder);
  NS_ENSURE_SUCCESS(rv, rv);
  // once we have the folder uri, open the db and search for the message id.
  nsCOMPtr <nsIMsgDatabase> msgDB;
  containingFolder->GetMsgDatabase(nsnull, getter_AddRefs(msgDB));
  nsCOMPtr<nsIMsgDBHdr> msgHdr;
  if (msgDB)
    msgDB->GetMsgHdrForMessageID(messageId.get(), getter_AddRefs(msgHdr));
  if (msgHdr)
  {
    nsMsgKey msgKey;
    msgHdr->GetMessageKey(&msgKey);
    rv = OpenMessengerWindowWithUri("mail:messageWindow", folderUri.get(), msgKey);  
    return rv;
  }
  return NS_ERROR_FAILURE;
}

NS_IMETHODIMP nsMessengerBootstrap::OpenMessengerWindowWithUri(const char *windowType, const char * aFolderURI, nsMsgKey aMessageKey)
{
  PRBool standAloneMsgWindow = PR_FALSE;
  nsCAutoString chromeUrl("chrome://messenger/content/");
  if (windowType && !strcmp(windowType, "mail:messageWindow"))
  {
    chromeUrl.Append("messageWindow.xul");
    standAloneMsgWindow = PR_TRUE;
  }
  nsCOMPtr<nsISupportsArray> argsArray;
  nsresult rv = NS_NewISupportsArray(getter_AddRefs(argsArray));
  NS_ENSURE_SUCCESS(rv, rv);

  // create scriptable versions of our strings that we can store in our nsISupportsArray....
  if (aFolderURI)
  {
    if (standAloneMsgWindow)
    {
      nsCOMPtr <nsIMsgFolder> folder;
      rv = GetExistingFolder(nsDependentCString(aFolderURI), getter_AddRefs(folder));
      NS_ENSURE_SUCCESS(rv, rv);
      NS_ENSURE_TRUE(folder, NS_ERROR_FAILURE);
      nsXPIDLCString msgUri;
      folder->GetBaseMessageURI(msgUri);

      nsCOMPtr<nsISupportsCString> scriptableMsgURI (do_CreateInstance(NS_SUPPORTS_CSTRING_CONTRACTID));
      NS_ENSURE_TRUE(scriptableMsgURI, NS_ERROR_FAILURE);
      msgUri.Append('#');
      msgUri.AppendInt(aMessageKey, 10);
      scriptableMsgURI->SetData(msgUri);
      argsArray->AppendElement(scriptableMsgURI);
      
    }
    nsCOMPtr<nsISupportsCString> scriptableFolderURI (do_CreateInstance(NS_SUPPORTS_CSTRING_CONTRACTID));
    NS_ENSURE_TRUE(scriptableFolderURI, NS_ERROR_FAILURE);

    scriptableFolderURI->SetData(nsDependentCString(aFolderURI));
    argsArray->AppendElement(scriptableFolderURI);

    if (!standAloneMsgWindow)
    {
      nsCOMPtr<nsISupportsPRUint32> scriptableMessageKey (do_CreateInstance(NS_SUPPORTS_PRUINT32_CONTRACTID));
      NS_ENSURE_TRUE(scriptableMessageKey, NS_ERROR_FAILURE);
      scriptableMessageKey->SetData(aMessageKey);
      argsArray->AppendElement(scriptableMessageKey);
    }
  }
  
  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
  NS_ENSURE_SUCCESS(rv, rv);

  // we need to use the "mailnews.reuse_thread_window2" pref
  // to determine if we should open a new window, or use an existing one.
  nsCOMPtr<nsIDOMWindow> newWindow;
  return wwatch->OpenWindow(0, chromeUrl.get(), "_blank",
                            "chrome,all,dialog=no", argsArray,
                             getter_AddRefs(newWindow));
}

// based off of nsMessengerBootstrap::HandleIndexerResult() from tb3
nsresult
nsMessengerBootstrap::HandleIndexerResult(const nsString &aEscapedPath)
{
#ifdef XP_MACOSX
  nsresult rv;

  // step 1:
  // convert
  // file://localhost/Users/sspitzer/Library/Caches/Metadata/Postbox/ImapMail/imap.googlemail.com/%5BGmail%5D.sbd/Sent%20Mail.mozmsgs/4A4E5049.3010300%2540sspitzer.org.pbeml
  // into
  // file://localhost/Users/sspitzer/Library/Caches/Metadata/Postbox/ImapMail/imap.googlemail.com/[Gmail].sbd/Sent Mail.mozmsgs/4A4E5049.3010300%40sspitzer.org.pbeml
  nsCAutoString unescapedFileSpec;
  NS_CopyUnicodeToNative(aEscapedPath, unescapedFileSpec);

  // if we are launching the .pbeml fle manually (like from Spotlight > Show All), it will look like
  // /Users/sspitzer/Library/Caches/Metadata/Postbox/Mail/Local Folders/fff.mozmsgs/4B4E3388.6010003%40postbox-inc.com.pbeml
  // notice, no file:// and it is already unescaped.
  //
  // if we are launching the .pbeml file from an app like Things, it will look like:
  // x-postbox-message://4B5BF8EB.4080102%2540gmail.com.pbeml
  PRInt32 xPostboxMessageIndex = aEscapedPath.Find(NS_LITERAL_STRING("x-postbox-message://")); 
  PRInt32 fileProtocolIndex = aEscapedPath.Find(NS_LITERAL_STRING("file://"));
  if (fileProtocolIndex == 0 || xPostboxMessageIndex == 0)
    NS_UnescapeURL(unescapedFileSpec);
  nsString aPath;
  NS_CopyNativeToUnicode(unescapedFileSpec, aPath);
  
  if (xPostboxMessageIndex == 0)
  {
    aPath.Cut(0, strlen("x-postbox-message://"));
    nsCAutoString messageId;
    NS_CopyUnicodeToNative(aPath, messageId);
    
    nsCOMPtr<nsIMsgGlobalIndex> globalIndex = do_GetService("@mozilla.org/msg-global-index;1", &rv);
    NS_ENSURE_SUCCESS(rv, rv);
    
    nsCOMPtr<nsIMsgDBHdr> msgHdr;
    rv = globalIndex->GetMsgHdrForMessageID(messageId, getter_AddRefs(msgHdr));
    NS_ENSURE_SUCCESS(rv, rv);
    
    nsCOMPtr <nsIMsgFolder> folder;
    rv = msgHdr->GetFolder(getter_AddRefs(folder));
    NS_ENSURE_SUCCESS(rv, rv);

    nsCString folderUri;
    rv = folder->GetURI(folderUri);
    NS_ENSURE_SUCCESS(rv, rv);
    
    return OpenMessengerWindowForMessageId(folderUri, messageId);
  }
  
  // parse file name - get path to containing folder, and message-id of message we're looking for
  // Then, open that message (in a 3-pane window?)
  //
  // step 2a:
  // convert
  // file://localhost/Users/sspitzer/Library/Caches/Metadata/Postbox/ImapMail/imap.googlemail.com/[Gmail].sbd/Sent Mail.mozmsgs/4A4E5049.3010300%40sspitzer.org.pbeml
  // or
  // x-postbox-message:///Users/sspitzer/Library/Caches/Metadata/Postbox/ImapMail/imap.gmail.com/INBOX.mozmsgs/4B5BF8EB.4080102%2540gmail.com.pbeml
  // into
  // /Users/sspitzer/Library/Caches/Metadata/Postbox/ImapMail/imap.googlemail.com/[Gmail].sbd/Sent Mail
  //
  // or
  // step 2b:
  // convert
  // /Users/sspitzer/Library/Caches/Metadata/Postbox/Mail/Local Folders/fff.mozmsgs/4B4E3388.6010003%40postbox-inc.com.pbeml
  // into 
  // /Users/sspitzer/Library/Caches/Metadata/Postbox/Mail/Local Folders/fff
  PRInt32 mozmsgsIndex = aPath.Find(NS_LITERAL_STRING(".mozmsgs"));
  nsString folderPathStr;
  aPath.Left(folderPathStr, mozmsgsIndex);
  if (fileProtocolIndex == 0)
    folderPathStr.Cut(0, strlen("file://localhost"));

  // On Mac, the .pbeml files are stored outside the profile in
  // ~/Library/Caches/Metadata.  We have to replace this root with
  // the profile dir, so we can find the actual mail there.  NOTE:
  // this works for profiles not in ~/Library/Application Support/Postbox/Profiles.
  nsCOMPtr<nsIFile> homeDir;
  rv = NS_GetSpecialDirectory(NS_OS_HOME_DIR, getter_AddRefs(homeDir));
  NS_ENSURE_SUCCESS(rv, rv);
  // Postbox Spotlight data goes in ~/Library/Caches/Metadata/Postbox
  homeDir->Append(NS_LITERAL_STRING("Library"));
  homeDir->Append(NS_LITERAL_STRING("Caches"));
  homeDir->Append(NS_LITERAL_STRING("Metadata"));
  homeDir->Append(NS_LITERAL_STRING("Postbox"));
  nsAutoString metadataDirStr;
  rv = homeDir->GetPath(metadataDirStr);
  NS_ENSURE_SUCCESS(rv, rv);

  nsCOMPtr<nsIFile> profileDir;
  rv = NS_GetSpecialDirectory("ProfD", getter_AddRefs(profileDir));
  NS_ENSURE_SUCCESS(rv, rv);

  nsCOMPtr<nsIIOService> ioService(do_GetService("@mozilla.org/network/io-service;1"));
  nsCOMPtr<nsIProtocolHandler> ph;
  ioService->GetProtocolHandler("file", getter_AddRefs(ph));
  nsCOMPtr<nsIFileProtocolHandler> fph(do_QueryInterface(ph));

  nsCAutoString profileDirStr;
  rv = fph->GetURLSpecFromFile(profileDir, profileDirStr);
  NS_ENSURE_SUCCESS(rv, rv);

  // Swap the Spotlight metadata root and the profile dir root
  // step 3a:
  // convert
  // /Users/sspitzer/Library/Caches/Metadata/Postbox/ImapMail/imap.googlemail.com/[Gmail].sbd/Sent Mail
  // into
  // file:///Users/sspitzer/Library/Application%20Support/Postbox/Profiles/xi45nfcr.gmail/ImapMail/imap.googlemail.com/[Gmail].sbd/Sent Mail
  //
  // This is the format that MsgMailboxGetURI() requires.
  folderPathStr.Replace(0, metadataDirStr.Length() + 1, NS_ConvertASCIItoUTF16(profileDirStr)); 

  // step 3b:
  // convert
  // file:///Users/sspitzer/Library/Application%20Support/Postbox/Profiles/cedrhzed.default/Mail/Local Folders/ccc
  // into
  // file:///Users/sspitzer/Library/Application%20Support/Postbox/Profiles/cedrhzed.default/Mail/Local%20Folders/ccc
  //
  // We lost the %20 in the hostname (Local%20Folders) in step 1.
  // This is the format that MsgMailboxGetURI() requires.
  PRInt32 mailLocalFoldersIndex = folderPathStr.Find(NS_LITERAL_STRING("/Mail/Local Folders/"));
  if (mailLocalFoldersIndex != -1)
  {
    folderPathStr.Replace(mailLocalFoldersIndex, strlen("/Mail/Local Folders/"), NS_LITERAL_STRING("/Mail/Local%20Folders/"));
  }
  
  nsCString folderUri;
  // need to convert to 8 bit chars...i.e., a local path.
  nsCAutoString nativeArg;
  NS_CopyUnicodeToNative(folderPathStr, nativeArg);
  rv = MsgMailboxGetURI(nativeArg.get(), folderUri);
  NS_ENSURE_SUCCESS(rv, rv);

  // step 4:
  // convert 
  // .mozmsgs/4A4E5049.3010300%40sspitzer.org.pbeml
  // into
  // 4A4E5049.3010300@sspitzer.org.pbeml
  //
  // This is the format that OpenMessengerWindowForMessageId() requires.
  nsAutoString unicodeMessageId;
  // strip off .pbeml the end as well
  aPath.Mid(unicodeMessageId, mozmsgsIndex + strlen(".mozmsgs/"), aPath.Length() - (strlen(".mozmsgs/") + mozmsgsIndex + strlen(".pbeml")));
  nsCAutoString messageId;
  NS_CopyUnicodeToNative(unicodeMessageId, messageId);
  // unescape the messageId
  NS_UnescapeURL(messageId);
  
  return OpenMessengerWindowForMessageId(folderUri, messageId);
#else
  return NS_OK;
#endif
}
